@@ -1,6 +1,6 @@
 
 /*
- * $Id: HttpMsg.cc,v 1.44 2007/12/21 23:50:24 hno Exp $
+ * $Id$
  *
  * DEBUG: section 74    HTTP Message
  * AUTHOR: Alex Rousskov
@@ -309,6 +309,14 @@ HttpMsg::httpMsgParseError()
     return -1;
 }
 
+void
+HttpMsg::setContentLength(int64_t clen)
+{
+    header.delById(HDR_CONTENT_LENGTH); // if any
+    header.putInt64(HDR_CONTENT_LENGTH, clen);
+    content_length = clen;
+}
+
 /* returns true if connection should be "persistent"
  * after processing this message */
 int
@@ -445,14 +453,16 @@ HttpParserRequestLen(HttpParser *hp)
 }
 #endif
 
-/*
+/**
  * Attempt to parse the request line.
  *
  * This will set the values in hmsg that it determines. One may end up
  * with a partially-parsed buffer; the return value tells you whether
  * the values are valid or not.
  *
- * @return 1 if parsed correctly, 0 if more is needed, -1 if error
+ * \retval	1 if parsed correctly
+ * \retval	0 if more is needed
+ * \retval	-1 if error
  *
  * TODO:
  *   * have it indicate "error" and "not enough" as two separate conditions!
@@ -463,7 +473,7 @@ HttpParserParseReqLine(HttpParser *hmsg)
 {
     int i = 0;
     int retcode = 0;
-    int maj = -1, min = -1;
+    unsigned int maj = 0, min = 0;
     int last_whitespace = -1, line_end = -1;
 
     debugs(74, 5, "httpParserParseReqLine: parsing " << hmsg->buf);
@@ -570,10 +580,14 @@ HttpParserParseReqLine(HttpParser *hmsg)
 
             /* next should be 1 or more digits */
             maj = 0;
-            for (; i < hmsg->req_end && (isdigit(hmsg->buf[i])); i++) {
+            for (; i < hmsg->req_end && (isdigit(hmsg->buf[i])) && maj < 65536; i++) {
                 maj = maj * 10;
                 maj = maj + (hmsg->buf[i]) - '0';
             }
+            if (maj >= 65536) {
+                retcode = -1;
+                goto finish;
+            }
             if (i >= hmsg->req_end) {
                 retcode = 0;
                 goto finish;
@@ -592,11 +606,16 @@ HttpParserParseReqLine(HttpParser *hmsg)
             /* next should be one or more digits */
             i++;
             min = 0;
-            for (; i < hmsg->req_end && (isdigit(hmsg->buf[i])); i++) {
+            for (; i < hmsg->req_end && (isdigit(hmsg->buf[i])) && min < 65536; i++) {
                 min = min * 10;
                 min = min + (hmsg->buf[i]) - '0';
             }
 
+            if (min >= 65536) {
+                retcode = -1;
+                goto finish;
+            }
+
             /* Find whitespace, end of version */
             hmsg->v_end = i;
             hmsg->u_end = last_whitespace - 1;
@@ -607,8 +626,6 @@ HttpParserParseReqLine(HttpParser *hmsg)
      * Rightio - we have all the schtuff. Return true; we've got enough.
      */
     retcode = 1;
-    assert(maj != -1);
-    assert(min != -1);
 
 finish:
     hmsg->v_maj = maj;
