@@ -8,9 +8,9 @@
 
 #include "BodyPipe.h"
 #include "adaptation/Initiate.h"
-#include "adaptation/Service.h"
 #include "adaptation/Message.h"
 #include "adaptation/ecap/MessageRep.h"
+#include "adaptation/ecap/ServiceRep.h"
 #include <libecap/common/forward.h>
 #include <libecap/common/memory.h>
 #include <libecap/host/xaction.h>
@@ -72,6 +72,8 @@ public:
     virtual const char *status() const;
 
 protected:
+    Service &service();
+
     Adaptation::Message &answer();
 
     void dropVirgin(const char *reason);
@@ -82,6 +84,7 @@ protected:
 
 private:
     AdapterXaction theMaster; // the actual adaptation xaction we represent
+    Adaptation::ServicePointer theService; ///< xaction's adaptation service
 
     MessageRep theVirginRep;
     MessageRep *theCauseRep;
@@ -92,6 +95,7 @@ private:
     typedef enum { opUndecided, opOn, opComplete, opNever } OperationState;
     OperationState proxyingVb; // delivering virgin body from core to adapter
     OperationState proxyingAb; // delivering adapted body from adapter to core
+    int adaptHistoryId;        ///< adaptation history slot reservation
     bool canAccessVb;          // virgin BodyPipe content is accessible
     bool abProductionFinished; // whether adapter has finished producing ab
     bool abProductionAtEnd;    // whether adapter produced a complete ab
